/*
 * A pipe pager providing access to pipe content and navigation support.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#include "pipe_pager.h"
#include "pipe_object_server.h"



/* Initialise a pager for a pipe with a shared page mapper for moderating
   access to loaded pages. At first, no mapper will be configured: this must be
   done by requesting a region. */

PipePager::PipePager(PipePaging *paging, bool writing)
: Pager(NULL, writing ? L4RE_DS_MAP_FLAG_RW : L4RE_DS_MAP_FLAG_RO),
  _paging(paging), _writing(writing)
{
    /* Initialise the size of the paged region. */

    _size = _paging->region_size();
}

int PipePager::expected_items()
{
    return PipeObject_expected_items;
}

ipc_server_handler_type PipePager::handler()
{
    return (ipc_server_handler_type) handle_PipeObject;
}



/* Close the pager, releasing the paging coordinator for the pipe. This will
   release all active page mappers. */

void PipePager::close()
{
    _paging->detach();
}

/* Support paging. */

long PipePager::map(offset_t offset, address_t hot_spot, flags_t flags, l4_snd_fpage_t *region)
{
    return Pager::map(offset, hot_spot, flags, region);
}



/* Return details of the current region. */

long PipePager::current_region(offset_t *populated_size, offset_t *size)
{
    if (_mapper != NULL)
    {
        *populated_size = _mapper->get_data_size();
        *size = _size;
        return L4_EOK;
    }
    else
        return -L4_EIO;
}

/* Obtain the next region and its details. */

long PipePager::next_region(offset_t *populated_size, offset_t *size)
{
    /* Obtain a new region if writing. */

    if (_writing)
        return next_region_for_writer(populated_size, size);
    else
        return next_region_for_reader(populated_size, size);
}

long PipePager::next_region_for_reader(offset_t *populated_size, offset_t *size)
{
    PageMapper *mapper = _paging->next_region();

    if (mapper == NULL)
        return -L4_EIO;

    _mapper = mapper;

    return current_region(populated_size, size);
}

long PipePager::next_region_for_writer(offset_t *populated_size, offset_t *size)
{
    /* Set the populated size before moving on. */

    if (_mapper != NULL)
        _mapper->set_data_size(*populated_size);

    /* Obtain the page mapper for the region. */

    PageMapper *mapper = _paging->add_region();

    if (mapper == NULL)
        return -L4_EIO;

    _mapper = mapper;

    return current_region(populated_size, size);
}

// vim: tabstop=4 expandtab shiftwidth=4
